@@ -6,7 +6,7 @@ from exam.cases import Exam
 
 from describe import expect
 
-from dummy import get_thing, get_it
+from dummy import get_thing, get_it, get_prop, ThingClass
 
 
 class SimpleTestCase(object):
@@ -65,6 +65,23 @@ class CaseWithBeforeHook(BaseTestCase):
         self.calls.append('run before')
 
 
+class CaseWithDecoratedBeforeHook(BaseTestCase):
+
+    def setup_some_state(self):
+        self.state = True
+
+    def setup_some_other_state(self):
+        self.other_state = True
+
+    @before(setup_some_state)
+    def should_have_run_before(self):
+        pass
+
+    @before(setup_some_state, setup_some_other_state)
+    def should_have_run_both_states(self):
+        pass
+
+
 class SubclassWithBeforeHook(CaseWithBeforeHook):
 
     @before
@@ -117,6 +134,17 @@ class SubclassedCaseWithPatcher(CaseWithPatcher):
     pass
 
 
+class CaseWithPatcherObject(BaseTestCase):
+
+    @patcher.object(ThingClass, 'prop')
+    def dummy_thing(self):
+        return 15
+
+
+class SubclassedCaseWithPatcherObject(CaseWithPatcherObject):
+    pass
+
+
 # TODO: Make the subclass checking just be a subclass of the test case
 class TestExam(Exam, TestCase):
 
@@ -130,6 +158,20 @@ class TestExam(Exam, TestCase):
         case.run()
         expect(case.calls_before_run).to == ['run before']
 
+    def test_before_decorator_runs_func_before_function(self):
+        case = CaseWithDecoratedBeforeHook()
+        expect(hasattr(case, 'state')).to == False
+        case.should_have_run_before()
+        expect(case.state).to == True
+
+    def test_before_decorator_runs_multiple_funcs(self):
+        case = CaseWithDecoratedBeforeHook()
+        expect(hasattr(case, 'state')).to == False
+        expect(hasattr(case, 'other_state')).to == False
+        case.should_have_run_both_states()
+        expect(case.state).to == True
+        expect(case.other_state).to == True
+
     def test_after_adds_each_method_after_test_case(self):
         case = CaseWithAfterHook()
         expect(case.calls).to == []
@@ -208,3 +250,23 @@ class TestExam(Exam, TestCase):
 
         case.cleanups[0]()
         expect(get_thing()).to != 12
+
+    def test_patcher_object_patches_object(self):
+        case = CaseWithPatcherObject()
+        expect(get_prop()).to != 15
+
+        case.run()
+        expect(get_prop()).to == 15
+
+        [cleanup() for cleanup in case.cleanups]
+        expect(get_prop()).to != 15
+
+    def test_patcher_object_works_with_subclasses(self):
+        case = SubclassedCaseWithPatcherObject()
+
+        expect(get_prop()).to != 15
+        case.run()
+        expect(get_prop()).to == 15
+
+        [cleanup() for cleanup in case.cleanups]
+        expect(get_prop()).to != 15
